#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import OpenRTM_aist
import DroneFlightDirector

import sys
from collections import namedtuple

from mock import Mock
from mock import patch, MagicMock


EXECUTE_RATE = 4
INPORT_SET_NUM = 3
DRONE_NUM = 3 #制御するドローン数


class TestDroneFlightDirector(unittest.TestCase):
    def setUp(self):
        self.m = DroneFlightDirector.DroneFlightDirector(0)

        self.m._Drone_Num = DRONE_NUM
        self.m._execute_rate = EXECUTE_RATE

        self.xx_logger = Mock()
        DroneFlightDirector.logger.warning = self.xx_logger.warning
        DroneFlightDirector.logger.error = self.xx_logger.error
        DroneFlightDirector.logger.info = self.xx_logger.info
        DroneFlightDirector.logger.debug = self.xx_logger.debug


    def test_check_outport_exe(self):

        in_state = [0] * INPORT_SET_NUM
        in_home = [0] * INPORT_SET_NUM
        in_drone = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM


        in_landing_flag = [False, False, False, False, False, False, False, False, False, False, True]

        in_state[0] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                       "Hovering", "Hovering", "Hovering", "RTL", "Disarm", "Hovering"]
        in_state[1] = ["Hovering", "RTL", "Hovering", "Hovering", "Hovering",
                       "Hovering", "Hovering", "Hovering", "Disarm", "RTL", "Hovering"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                       "RTL", "Disarm", "RTL", "Disarm", "RTL", "Hovering"]

        in_home[0] = [True, True, True, True, False, True, True, False, True, True, True]
        in_home[1] = [True, True, True, True, True, True, True, True, True, True, True]
        in_home[2] = [True, True, True, True, True, True, True, True, True, True, True]
 
        in_drone[0] = [True, True, True, True, True, True, True, True, True, True, True]
        in_drone[1] = [True, True, True, False, True, True, False, True, True, True, True]
        in_drone[2] = [True, True, True, True, True, True, True, True, True, True, True]

        in_mag[0] = [True, True, True, True, True, False, True, True, True, True, True]
        in_mag[1] = [True, True, True, True, True, True, True, True, True, True, True]
        in_mag[2] = [True, True, False, True, True, True, True, True, True, True, True]

        result = [True, True, False, False, False, False, False, False, False, False, False]



        test_case = 0
        while test_case < len(result):
            self.m._landing_flag = in_landing_flag[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._home_gps[drone_num] = in_home[drone_num][test_case]
                self.m._drone_gps[drone_num] = in_drone[drone_num][test_case]
                self.m._mag[drone_num] = in_mag[drone_num][test_case]
                drone_num += 1

            self.assertEqual(self.m._check_outport_exe(), result[test_case])
            test_case += 1

    def test_is_set_to_home(self):
        xx = Mock()
        self.m._is_all_drone_on_home = xx._is_all_drone_on_home

        in_all_rtl = [False, False, True, True, True]
        in_landing_flag = [False, True, False, True, True]
        in_all_drone_home = [None, None, None, False, True]

        result = [False, False, True, False, True]


        test_case = 0
        while test_case < len(result):
            self.m._all_rtl = in_all_rtl[test_case]
            self.m._landing_flag = in_landing_flag[test_case]
            xx._is_all_drone_on_home.return_value = in_all_drone_home[test_case]

            self.assertEqual(self.m._is_set_to_home(), result[test_case])
            test_case += 1

    def test_is_all_drone_on_home(self):

        in_state = [0] * INPORT_SET_NUM
        in_home_lat = [0] * INPORT_SET_NUM
        in_home_lon = [0] * INPORT_SET_NUM
        in_home_alt = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM


        in_state[0] = ["RTL", "RTL", "RTL", "RTL", "RTL", "RTL",
                       "Disarm", "Disarm", "Disarm", "Disarm", "Disarm", "Disarm",
                       "Disarm", "Disarm", "Disarm", "Disarm", "Disarm", "Disarm"]
        in_state[1] = ["RTL", "RTL", "RTL", "RTL", "RTL", "RTL",
                       "RTL", "RTL", "RTL", "RTL", "RTL", "RTL",
                       "Disarm", "Disarm", "Disarm", "Disarm", "Disarm", "Disarm"]
        in_state[2] = ["RTL", "RTL", "RTL", "RTL", "RTL", "RTL",
                       "RTL", "RTL", "RTL", "RTL", "RTL", "RTL",
                       "RTL", "RTL", "RTL", "RTL", "RTL", "RTL"]

        in_home_lat[0] = [37.500000, 37.500000, 37.500000, 37.500000, 37.500000, 37.500000,
                          37.500000, 37.500000, 37.500000, 37.500000, 37.500000, 37.500000,
                          37.500000, 37.500000, 37.500000, 37.500000, 37.500000, 37.500000]
        in_home_lat[1] = [37.510000, 37.510000, 37.510000, 37.510000, 37.510000, 37.510000,
                          37.510000, 37.510000, 37.510000, 37.510000, 37.510000, 37.510000,
                          37.510000, 37.510000, 37.510000, 37.510000, 37.510000, 37.510000]
        in_home_lat[2] = [37.520000, 37.520000, 37.520000, 37.520000, 37.520000, 37.520000,
                          37.520000, 37.520000, 37.520000, 37.520000, 37.520000, 37.520000,
                          37.520000, 37.520000, 37.520000, 37.520000, 37.520000, 37.520000]

        in_home_lon[0] = [139.900000, 139.900000, 139.900000, 139.900000, 139.900000, 139.900000,
                          139.900000, 139.900000, 139.900000, 139.900000, 139.900000, 139.900000,
                          139.900000, 139.900000, 139.900000, 139.900000, 139.900000, 139.900000]
        in_home_lon[1] = [139.910000, 139.910000, 139.910000, 139.910000, 139.910000, 139.910000,
                          139.910000, 139.910000, 139.910000, 139.910000, 139.910000, 139.910000,
                          139.910000, 139.910000, 139.910000, 139.910000, 139.910000, 139.910000]
        in_home_lon[2] = [139.920000, 139.920000, 139.920000, 139.920000, 139.920000, 139.920000,
                          139.920000, 139.920000, 139.920000, 139.920000, 139.920000, 139.920000,
                          139.920000, 139.920000, 139.920000, 139.920000, 139.920000, 139.920000]
        in_home_alt[0] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[2] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0]


        in_drone_lat[0] = [37.500010, 37.500002, 37.500004, 37.500005, 37.500007, 37.500016,
                           37.500010, 37.500002, 37.500004, 37.500005, 37.500007, 37.500016,
                           37.500010, 37.500002, 37.500004, 37.500005, 37.500007, 37.500016]
        in_drone_lat[1] = [37.510040, 37.510020, 37.510006, 37.510008, 37.510005, 37.510004,
                           37.510040, 37.510020, 37.510006, 37.510008, 37.510005, 37.510004,
                           37.510040, 37.510020, 37.510006, 37.510008, 37.510005, 37.510004]
        in_drone_lat[2] = [37.520005, 37.520050, 37.520020, 37.520010, 37.520003, 37.520006,
                           37.520005, 37.520050, 37.520020, 37.520010, 37.520003, 37.520006,
                           37.520005, 37.520050, 37.520020, 37.520010, 37.520003, 37.520006]
        in_drone_lon[0] = [139.900020, 139.900008, 139.900006, 139.900010, 139.900005, 139.900009,
                           139.900020, 139.900008, 139.900006, 139.900010, 139.900005, 139.900009,
                           139.900020, 139.900008, 139.900006, 139.900010, 139.900005, 139.900009]
        in_drone_lon[1] = [139.910050, 139.910030, 139.910007, 139.910010, 139.910011, 139.910008,
                           139.910050, 139.910030, 139.910007, 139.910010, 139.910011, 139.910008,
                           139.910050, 139.910030, 139.910007, 139.910010, 139.910011, 139.910008]
        in_drone_lon[2] = [139.920005, 139.920010, 139.920020, 139.920006, 139.920021, 139.920014,
                           139.920005, 139.920010, 139.920020, 139.920006, 139.920021, 139.920014,
                           139.920005, 139.920010, 139.920020, 139.920006, 139.920021, 139.920014]
        in_drone_alt[0] = [8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                           8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                           8.0, 8.0, 8.0, 8.0, 8.0, 8.0]
        in_drone_alt[1] = [10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                           10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                           10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
        in_drone_alt[2] = [12.0, 12.0, 12.0, 12.0, 12.0, 12.0,
                           12.0, 12.0, 12.0, 12.0, 12.0, 12.0,
                           12.0, 12.0, 12.0, 12.0, 12.0, 12.0]

        result = [False, False, False, True, True, True,
                  False, False, False, True, True, True,
                  True, False, False, True, True, True]



        test_case = 0
        while test_case < len(result):

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]

                self.m._d_HomeGPSIn[drone_num].latitude = in_home_lat[drone_num][test_case]
                self.m._d_HomeGPSIn[drone_num].longitude = in_home_lon[drone_num][test_case]
                self.m._d_HomeGPSIn[drone_num].altitude = in_home_alt[drone_num][test_case]

                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].altitude = in_drone_alt[drone_num][test_case]

                drone_num += 1

            self.assertEqual(self.m._is_all_drone_on_home(), result[test_case])

            test_case += 1

    def test_set_leader(self):

        in_state = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        in_state[0] = ["Hovering", "Hovering", "Hovering", "Disarm", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_state[2] = ["Hovering", "Hovering", "Disarm", "RTL", "Disarm"]

        in_accel[0] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[1] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[2] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_leader_set_first = [True, False, False, False, False]

        in_rtl_num = [0, 0, 0, 1, 2]


        leader_first_result = [False, False, False, False, False]
        rtm_num_result = [0, 0, 1, 2, 2]


        test_case = 0
        while test_case < len(leader_first_result):

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._d_inAccel[drone_num].data = in_accel[drone_num][test_case]
                drone_num += 1

            self.m._leader_set_first = in_leader_set_first[test_case]
            self.m._rtl_num = in_rtl_num[test_case]

            self.m._set_leader()

            self.assertEqual(self.m._leader_set_first, leader_first_result[test_case])
            self.assertEqual(self.m._rtl_num, rtm_num_result[test_case])

            test_case += 1

    def test_calc_leader_pos(self):

        in_state = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        in_avg_drone_alt = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]
        in_state[0] = ["Hovering", "RTL", "Disarm", "Hovering", "Disarm", "RTL", "Hovering", "RTL", "Disarm"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm", "Hovering", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_alt[0] = [0.0, 0.0, 0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0]
        in_drone_alt[1] = [0.0, 0.0, 0.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
        in_drone_alt[2] = [0.0, 0.0, 0.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0]
        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]
        in_mag[0] = [320, 320, 320, 160, 160, 160, 270, 270, 270]
        in_mag[1] = [10, 10, 10, 190, 190, 190, 80, 80, 80]
        in_mag[2] = [120, 120, 120, 185, 185, 185, 100, 100, 100]
        in_rtl_num = [0, 1, 2, 0, 1, 2, 0, 1, 2]
        in_accel[0] = [[3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0],
                       [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0], [3.0, 0.0, 0.0]]
        in_accel[1] = [[2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0],
                       [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0], [2.0, 1.0, 0.0]]
        in_accel[2] = [[1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0],
                       [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0], [1.0, 3.0, 3.0]]

        avg_drone_lat_result = [37.530, 37.535, 37.540, 37.530, 37.535, 37.540, 37.530, 37.535, 37.540]
        avg_drone_lon_result = [139.940, 139.945, 139.950, 139.940, 139.945, 139.950, 139.940, 139.945, 139.950]
        avg_drone_alt_result = [7.0, 8.0, 9.0, 10.0, 11.0, 12.0, 10.0, 11.0, 12.0]
        avg_mag_result = [30, 65, 120, 178, 187, 185, 30, 90, 100]


        avg_speed_result = [2.799, 2.699, 3.162, 2.799, 2.699, 3.162, 2.799, 2.699, 3.162]

        test_case = 0
        while test_case < len(avg_drone_lat_result):
            self.m._avg_drone_pos.altitude = in_avg_drone_alt[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].altitude = in_drone_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._d_inMag[drone_num].data = in_mag[drone_num][test_case]

                self.m._d_inAccel[drone_num].data = in_accel[drone_num][test_case]
                drone_num += 1

            self.m._rtl_num = in_rtl_num[test_case]

            self.m._calc_leader_pos()

            self.assertEqual(self.m._avg_drone_pos.latitude, avg_drone_lat_result[test_case])
            self.assertEqual(self.m._avg_drone_pos.longitude, avg_drone_lon_result[test_case])
            self.assertEqual(self.m._avg_drone_pos.altitude, avg_drone_alt_result[test_case])
            self.assertEqual(self.m._avg_mag, avg_mag_result[test_case])
            self.assertEqual(round(self.m._avg_speed,3), avg_speed_result[test_case])

            test_case += 1


    def test_set_leader_pos(self):

        in_avg_lat = [37.530, 37.530]
        in_avg_lon = [139.940, 139.940]
        in_avg_alt = [7.0, 7.0]
        in_leader_lat = [37.540, 37.540]
        in_leader_lon = [139.950, 39.950]
        in_leader_alt = [5.0, 5.0]
        in_leader_set = [False, True]


        leader_lat_result = [37.530, 37.530]
        leader_lon_result = [139.940, 139.940]
        leader_alt_result = [5.0, 7.0]



        test_case = 0
        while test_case < len(leader_lat_result):
            self.m._avg_drone_pos.latitude = in_avg_lat[test_case]
            self.m._avg_drone_pos.longitude = in_avg_lon[test_case]
            self.m._avg_drone_pos.altitude = in_avg_alt[test_case]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_pos.altitude = in_leader_alt[test_case]
            self.m.leader_set = in_leader_set[test_case]

            self.m._set_leader_pos()

            self.assertEqual(self.m._leader_pos.latitude, leader_lat_result[test_case])
            self.assertEqual(self.m._leader_pos.longitude, leader_lon_result[test_case])
            self.assertEqual(self.m._leader_pos.altitude, leader_alt_result[test_case])

            test_case += 1

    def test_drone_diff(self):

        in_state = [0] * INPORT_SET_NUM
        in_diff_lat = [0] * DRONE_NUM
        in_diff_lon = [0] * DRONE_NUM
        in_diff_alt = [0] * DRONE_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_mag = [0] * INPORT_SET_NUM
        diff_lat_result = [0] * DRONE_NUM
        diff_lon_result = [0] * DRONE_NUM
        diff_alt_result = [0] * DRONE_NUM



        in_state[0] = ["Hovering", "RTL", "Disarm", "Disarm", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_diff_lat[0] = [-0.15, -0.15, -0.15, -0.15, -0.15]
        in_diff_lat[1] = [0.15, 0.15, 0.15, 0.15, 0.15]
        in_diff_lat[2] = [0.25, 0.25, 0.25, 0.25, 0.25]
        in_diff_lon[0] = [-0.15, -0.15, -0.15, -0.15, -0.15]
        in_diff_lon[1] = [0.15, 0.15, 0.15, 0.15, 0.15]
        in_diff_lon[2] = [0.25, 0.25, 0.25, 0.25, 0.25]
        in_diff_alt[0] = [-1.0, -1.0, -1.0, -1.0, -1.0]
        in_diff_alt[1] = [2.0, 2.0, 2.0, 2.0, 2.0]
        in_diff_alt[2] = [5.0, 5.0, 5.0, 5.0, 5.0]

        in_leader_lat = [37.530, 37.535, 37.540, 37.535, 37.540]
        in_leader_lon = [139.940, 139.945, 139.950, 139.945, 139.950]
        in_leader_alt = [7.0, 8.0, 9.0, 11.0, 12.0]

        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_alt[0] = [0.0, 0.0, 0.0, 8.0, 8.0]
        in_drone_alt[1] = [0.0, 0.0, 0.0, 10.0, 10.0]
        in_drone_alt[2] = [0.0, 0.0, 0.0, 12.0, 12.0]
        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0]
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0]


        diff_lat_result[0] = [-0.01, -0.15, -0.15, -0.15, -0.15]
        diff_lat_result[1] = [0.0, -0.005, 0.15, -0.005, 0.15]
        diff_lat_result[2] = [0.01, 0.005, 0.0, 0.005, 0.0]
        diff_lon_result[0] = [-0.01, -0.15, -0.15, -0.15, -0.15]
        diff_lon_result[1] = [0.0, -0.005, 0.15, -0.005, 0.15]
        diff_lon_result[2] = [0.01, 0.005, 0.0, 0.005, 0.0]
        diff_alt_result[0] = [-2.0, -1.0, -1.0, -1.0, -1.0]
        diff_alt_result[1] = [0.0, -1.0, 2.0, -1.0, 2.0]
        diff_alt_result[2] = [2.0, 1.0, 0.0, 1.0, 0.0]

        test_case = 0
        while test_case < len(diff_lat_result[0]):

            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_pos.altitude = in_leader_alt[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].latitude = in_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_diff_lon[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].altitude = in_diff_alt[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].altitude = in_drone_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                drone_num += 1


            self.m._drone_diff()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._drone_diff_pos[drone_num].latitude,3), diff_lat_result[drone_num][test_case])
                self.assertEqual(round(self.m._drone_diff_pos[drone_num].longitude,3), diff_lon_result[drone_num][test_case])
                self.assertEqual(self.m._drone_diff_pos[drone_num].altitude, diff_alt_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_set_pivot(self):

        in_state = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM

        in_state[0] = ["Hovering", "RTL", "Disarm", "Hovering", "Disarm", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]
        in_leader_lat = [37.530, 37.535, 37.540, 37.530, 37.535, 37.540]
        in_leader_lon = [139.950, 139.960, 139.980, 139.950, 139.960, 139.980]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.980, 139.980, 139.980, 139.980, 139.980, 139.980]
        in_avg_mag = [350, 350, 350, 160, 160, 160]

        right_pivot_result = [2, 2, 2, 0, 1, 2]
        left_pivot_result = [0, 1, 2, 2, 2, 2]
       

        test_case = 0
        while test_case < len(right_pivot_result):

            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]

                drone_num += 1

            self.m._avg_drone_pos.latitude = in_leader_lat[test_case]
            self.m._avg_drone_pos.longitude = in_leader_lon[test_case]
            self.m._avg_mag = in_avg_mag[test_case]

            self.m._set_pivot()

            self.assertEqual(self.m._right_pivot, right_pivot_result[test_case])
            self.assertEqual(self.m._left_pivot, left_pivot_result[test_case])

            test_case += 1


    def test_diff_target_angle(self):
        in_avg_mag = [30, 30, 280, 280]
        in_target_lat = [37.520, 37.580, 37.520, 37.580]
        in_target_lon = [139.910, 139.810, 139.910, 139.810]
        in_pos_lat = [37.550, 37.500, 37.550, 37.500]
        in_pos_lon = [139.940, 139.890, 139.940, 139.890]

        target = namedtuple("target", ["latitude", "longitude"])
        pos = namedtuple("pos", ["latitude", "longitude"])

        deg_result = [-165.0, -75.0, -55.0, 35.0]
       

        test_case = 0
        while test_case < len(deg_result):

            self.m._avg_mag = in_avg_mag[test_case]

            t = target(in_target_lat[test_case], in_target_lon[test_case])
            p = pos(in_pos_lat[test_case], in_pos_lon[test_case])
            deg = round(self.m._diff_target_angle(t, p), 1)

            self.assertEqual(deg, deg_result[test_case])

            test_case += 1
    
    def test_set_target_gps(self):
        xx = Mock()
        self.m._takeoff_landing = xx._takeoff_landing
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._turn_dir = xx._turn_dir

        in_state = [0] * INPORT_SET_NUM
        in_home_lat = [0] * INPORT_SET_NUM
        in_home_lon = [0] * INPORT_SET_NUM
        in_home_alt = [0] * INPORT_SET_NUM
        in_takeoff_alt = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_accel = [0] * INPORT_SET_NUM

        target_lat_result = [0] * INPORT_SET_NUM
        target_lon_result = [0] * INPORT_SET_NUM
        target_alt_result = [0] * INPORT_SET_NUM


        in_diff_lat = [0] * DRONE_NUM
        in_diff_lon = [0] * DRONE_NUM
        in_diff_alt = [0] * DRONE_NUM



        in_takeoff_landing = [1, -1, 0, 0, 0, 0, 0, 0, 0, 0,
                              0,  0, 0, 0, 0, 0, 0, 0]

        in_state[0] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "RTL", "RTL", "RTL", 
                         "Disarm", "Disarm", "Disarm", "Disarm", "RTL", "RTL", "RTL", "Hovering"]
        in_state[1] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", 
                         "Hovering", "Hovering", "Disarm", "Disarm", "Disarm", "RTL", "RTL", "Hovering"]
        in_state[2] =["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", 
                         "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering"]

        in_home_lat[0] = [37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 
                          37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500]
        in_home_lat[1] = [37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510,
                          37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510, 37.510]
        in_home_lat[2] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                          37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]

        in_home_lon[0] = [139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900,
                          139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900]
        in_home_lon[1] = [139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910,
                          139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910, 139.910]
        in_home_lon[2] = [139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920,
                          139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920, 139.920]
        in_home_alt[0] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_home_alt[2] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]

        in_takeoff_alt[0] = [5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0,
                             5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0, 5.0]
        in_takeoff_alt[1] = [7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0,
                             7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0, 7.0]
        in_takeoff_alt[2] = [9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0,
                             9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0, 9.0]

        in_leader_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530000, 
                         37.535, 37.535, 37.535, 37.535, 37.535, 
                         37.540, 37.540, 37.540, 37.540, 37.540, 37.530]
        in_leader_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 
                         139.960, 139.960, 139.960, 139.960, 139.960, 
                         139.980, 139.980, 139.980, 139.980, 139.980, 139.950]
        in_leader_alt = [7.0, 10.0, 7.0, 10.0, 10.0, 10.0, 10.0, 11.0, 10.0, 11.0,
                           11.0, 11.0, 12.0, 12.0, 12.0, 11.0, 12.0, 12.0]

        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                           37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                           37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540,
                           37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                           139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                           139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980,
                           139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980, 139.980]
        in_drone_alt[0] = [8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                           8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0]
        in_drone_alt[1] = [10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0,
                           10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0]
        in_drone_alt[2] = [12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0,
                           12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0, 12.0]

        in_front_back = [0, 0, 1, 0, 0, 0, 0, 1, 0, 0,
                        0, 0, 1, 0, 0, 0, 0, 0]
        in_right_left = [0, 0, 0, -1, 0, 0, 0, 0, -1, 0,
                        0, 0, 0, -1, 0, 0, 0, 0]
        in_up_down = [0, 0, 0, 0, 1, 0, 0, 0, 0, 1,
                        0, 0, 0, 0, 1, 0, 0, 0]

        in_turn_dir = [0, 0, 0, 0, 0, 1, -1, 0, 0, 0,
                       1, -1, 0, 0, 0, 1, -1, 0]

        #in_avg_mag = [350, 350, 350, 160, 160, 160]

        in_mode = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0]
        in_accel[0] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[1] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]
        in_accel[2] = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0],
                       [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]]

        in_last_front_back = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                              0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_last_right_left = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                              0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        in_last_up_down = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]


        in_diff_lat[0] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, -0.01 ]
        in_diff_lat[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                          -0.005, -0.005, -0.005, -0.005, -0.005,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0 ]
        in_diff_lat[2] = [0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
                          0.005, 0.005, 0.005, 0.005, 0.005,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.01 ]
        in_diff_lon[0] = [-0.02, -0.02, -0.02, -0.02, -0.02, -0.02, -0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, -0.02 ]
        in_diff_lon[1] = [-0.01, -0.01, -0.01, -0.01, -0.01, -0.01, -0.01,
                          -0.02, -0.02, -0.02, -0.02, -0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0, -0.01 ]
        in_diff_lon[2] = [0.03, 0.03, 0.03, 0.03, 0.03, 0.03, 0.03,
                          0.02, 0.02, 0.02, 0.02, 0.02,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.03 ]
        in_diff_alt[0] = [-2.0, -2.0, -2.0, -2.0, -2.0, -2.0, -2.0, 
                           0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, -2.0 ]
        in_diff_alt[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                          -1.0, -1.0, -1.0, -1.0, -1.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0 ]
        in_diff_alt[2] = [2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 
                          1.0, 1.0, 1.0, 1.0, 1.0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 2.0 ]

        in_rtl_num = [0, 0, 0, 0, 0, 0, 0,
                      1, 1, 1, 1, 1,
                      2, 2, 2, 2, 2,
                      0]

        target_lat_result[0] = [37.500, 37.500, 37.52041, 37.520, 37.520, 37.52003, 37.520, 37.500, 37.500, 37.500,
                           37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.520]

        target_lat_result[1] = [37.510, 37.510, 37.53041, 37.530, 37.530, 37.53002, 37.53001, 37.53041, 37.530, 37.530,
                           37.53003, 37.530, 37.510, 37.510, 37.510, 37.510, 37.510, 37.530]

        target_lat_result[2] = [37.520, 37.520, 37.54041, 37.540, 37.540, 37.540, 37.54003, 37.54041, 37.540, 37.540,
                           37.540, 37.54003, 37.54041, 37.540, 37.540, 37.540, 37.540, 37.540]

        target_lon_result[0] = [139.900, 139.900, 139.930, 139.92968, 139.930, 139.92999, 139.930, 139.900, 139.900, 139.900,
                           139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.930]

        target_lon_result[1] = [139.910, 139.910, 139.940, 139.93968, 139.940, 139.93999, 139.93999, 139.940, 139.93968, 139.940,
                           139.93999, 139.940, 139.910, 139.910, 139.910, 139.910, 139.910, 139.940]

        target_lon_result[2] = [139.920, 139.920, 139.980, 139.97968, 139.980, 139.980, 139.97999, 139.980, 139.97968, 139.980,
                           139.980, 139.97999, 139.980, 139.97968, 139.980, 139.980, 139.980, 139.980]

        target_alt_result[0] = [5.0, 0.0, 8.0, 8.0, 9.0, 8.0, 8.0, 0.0, 0.0, 0.0,
                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 8.0]

        target_alt_result[1] = [7.0, 0.0, 10.0, 10.0, 11.0, 10.0, 10.0, 10.0, 10.0, 11.0,
                                10.0, 10.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.0]

        target_alt_result[2] = [9.0, 0.0, 12.0, 12.0, 13.0, 12.0, 12.0, 12.0, 12.0, 13.0,
                                12.0, 12.0, 12.0, 12.0, 13.0, 12.0, 12.0, 12.0]


        test_case = 0
        while test_case < len(target_lat_result[0]):
            xx._takeoff_landing.return_value = in_takeoff_landing[test_case]
            xx._front_back.return_value =in_front_back[test_case]
            xx._right_left.return_value =in_right_left[test_case]
            xx._up_down.return_value =in_up_down[test_case]
            xx._turn_dir.return_value = in_turn_dir[test_case]

            self.m._rtl_num = in_rtl_num[test_case]

            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_pos.altitude = in_leader_alt[test_case]

            self.m._mode = in_mode[test_case]

            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]
            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._d_HomeGPSIn[drone_num].latitude = in_home_lat[drone_num][test_case]
                self.m._d_HomeGPSIn[drone_num].longitude = in_home_lon[drone_num][test_case]
                self.m._d_HomeGPSIn[drone_num].altitude = in_home_alt[drone_num][test_case]
                self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].altitude = in_drone_alt[drone_num][test_case]

                self.m._drone_diff_pos[drone_num].latitude = in_diff_lat[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].longitude = in_diff_lon[drone_num][test_case]
                self.m._drone_diff_pos[drone_num].altitude = in_diff_alt[drone_num][test_case]

                #target_gps.py[get_target_cal] if drone_target.altitude > 0.0を満たすため
                self.m._target[drone_num].altitude = 1.0
                self.m._turn_first = True
                self.m._avg_mag = 350
                self.m._d_inAccel[drone_num].data = in_accel[drone_num][test_case]

                drone_num += 1

            self.m._set_target_gps()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_TargetGPSOut[drone_num].latitude,5), target_lat_result[drone_num][test_case])
                self.assertEqual(round(self.m._d_TargetGPSOut[drone_num].longitude,5), target_lon_result[drone_num][test_case])
                self.assertEqual(self.m._d_TargetGPSOut[drone_num].altitude, target_alt_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

        

    def test_trans_target_speed(self):

        in_state = [0] * INPORT_SET_NUM
        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_target_lat = [0] * INPORT_SET_NUM
        in_target_lon = [0] * INPORT_SET_NUM
        in_target_alt = [0] * INPORT_SET_NUM

        target_speed_result = [0] * INPORT_SET_NUM


        in_speed = [3.0, 3.0, 3.0, 3.0]
        in_leader_lat = [37.530, 37.530, 37.530, 37.530]
        in_leader_lon = [139.940, 139.940, 139.940, 139.940]
        in_leader_target_lat = [37.535, 37.535, 37.535, 37.535]
        in_leader_target_lon = [139.950, 139.950, 139.950, 139.950]
        in_state[0] = ["Hovering", "Disarm", "RTL", "RTL"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering"]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.930, 139.930, 139.930, 139.930]
        in_drone_lon[2] = [139.960, 139.960, 139.960, 139.960]
        in_target_lat[0] = [37.525, 37.525, 37.525, 37.525]
        in_target_lat[1] = [37.534, 37.534, 37.534, 37.534]
        in_target_lat[2] = [37.547, 37.547, 37.547, 37.547]
        in_target_lon[0] = [139.940, 139.940, 139.940, 139.940]
        in_target_lon[1] = [139.940, 139.940, 139.940, 139.940]
        in_target_lon[2] = [139.969, 139.969, 139.969, 139.969]

        

        target_speed_result[0] = [3.0, 3.0, 3.0, 3.0]
        target_speed_result[1] = [2.843, 2.843, 2.843, 2.843]
        target_speed_result[2] = [3.196, 3.196, 3.196, 3.196]



        test_case = 0
        while test_case < len(target_speed_result[0]):
            self.m._SpeedParam = in_speed[test_case]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_target.latitude = in_leader_target_lat[test_case]
            self.m._leader_target.longitude = in_leader_target_lon[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_TargetGPSOut[drone_num].latitude = in_target_lat[drone_num][test_case]
                self.m._d_TargetGPSOut[drone_num].longitude = in_target_lon[drone_num][test_case]
                drone_num += 1


            self.m._trans_target_speed()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_outTargetSpeed[drone_num].data,3), target_speed_result[drone_num][test_case])
                drone_num += 1

            test_case += 1

    def test_turn_target_speed(self):

        init_target_speed = [0] * INPORT_SET_NUM

        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_state = [0] * INPORT_SET_NUM

        target_speed_result = [0] * INPORT_SET_NUM


        init_target_speed[0] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        init_target_speed[1] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        init_target_speed[2] = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]


        in_right_pivot = [0, 1, 2, 0, 0, 0]
        in_left_pivot = [2, 2, 2, 2, 1, 0]
        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.980, 139.980, 139.980, 39.980, 139.980, 139.980]
        in_turn_pivot = [2, 2, 2, 0, 0, 0]
        in_speed = [3.0, 3.0, 3.0, 3.0, 3.0, 3.0]
        in_state[0] = ["Hovering", "Disarm", "RTL", "Hovering", "Hovering", "Hovering"]
        in_state[1] = ["Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Disarm", "RTL"]


        target_speed_result[0] = [3.0,    0.0, 0.0,    0.0, 0.0, 0.0]
        target_speed_result[1] = [2.2477, 3.0, 0.0, 0.0005, 3.0, 0.0]
        target_speed_result[2] = [0.0,    0.0, 0.0,    3.0, 0.0, 0.0]


        test_case = 0
        while test_case < len(target_speed_result[0]):
            self.m._turn_pivot = in_turn_pivot[test_case]
            self.m._right_pivot = in_right_pivot[test_case]
            self.m._left_pivot = in_left_pivot[test_case]
            self.m._SpeedParam = in_speed[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]

                self.m._d_outTargetSpeed[drone_num].data = init_target_speed[drone_num][test_case]

                drone_num += 1

            self.m._turn_target_speed()

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.assertEqual(round(self.m._d_outTargetSpeed[drone_num].data,4), target_speed_result[drone_num][test_case])
                drone_num += 1

            test_case += 1


    def test_set_target_yaw(self):
        xx = Mock()
        self.m._turn_dir = xx._turn_dir

        init_target_yaw = [0] * INPORT_SET_NUM

        in_drone_lat = [0] * INPORT_SET_NUM
        in_drone_lon = [0] * INPORT_SET_NUM
        in_drone_alt = [0] * INPORT_SET_NUM
        in_target_lat = [0] * INPORT_SET_NUM
        in_target_lon = [0] * INPORT_SET_NUM
        in_target_alt = [0] * INPORT_SET_NUM
        in_state = [0] * INPORT_SET_NUM

        target_yaw_result = [0] * INPORT_SET_NUM


        init_target_yaw[0] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]
        init_target_yaw[1] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]
        init_target_yaw[2] = [[0, 315], [0, 315], [0, 315], [0, 315], [0, 135], [0, 135], [0, 135], [0, 135], 
                      [0, 315], [0, 135], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315], [0, 315],
                      [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135], [0, 135]]

        in_turn_dir = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
                       -1, -1, -1, 1, 1, 1, -1, -1, -1, 1, 1, 1,
                       0, 0]

        in_mode = [1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                   2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
                   0, 0]

        in_move_mode = ["Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Relative", "Relative",
                        "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", 
                        "Relative", "Relative", "Relative", "Relative", "Relative", "Relative",
                        "Absolute", "Relative"]



        in_leader_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                         37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                         37.530, 37.530, 37.530, 37.530]
        in_leader_lon = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                         139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                         139.940, 139.940, 139.940, 139.940]
        in_leader_target_lat = [37.535, 37.525, 37.530, 37.530, 37.535, 37.525, 37.530, 37.530, 37.535, 37.530,
                                37.535, 37.525, 37.530, 37.530, 37.535, 37.525, 37.530, 37.530, 37.535, 37.530,
                                37.535, 37.525, 37.530, 37.530]
        in_leader_target_lon = [139.940, 139.940, 139.945, 139.935, 139.940, 139.940, 139.945, 139.935, 139.940, 139.945,
                                139.940, 139.940, 139.945, 139.935, 139.940, 139.940, 139.945, 139.935, 139.940, 139.945,
                                139.940, 139.940, 139.945, 139.935]

        in_right_pivot = [2, 2, 2, 2, 0, 0, 0, 0, 2, 0, 
                          2, 2, 2, 2, 2, 2, 0, 1, 2, 0, 1, 2, 2, 0]
        in_left_pivot = [0, 0, 0, 0, 2, 2, 2, 2, 0, 2,
                         0, 1, 2, 0, 1, 2, 2, 2, 2, 2, 2, 2, 0, 2]

        in_turn_end = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                         2, 2, 2, 0, 1, 2, 0, 1, 2, 2, 2, 2, 0, 0]

        in_drone_lat[0] = [37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                           37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lat[1] = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530,
                           37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_drone_lat[2] = [37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540,
                           37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540]
        in_drone_lon[0] = [139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                           139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_lon[1] = [139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940,
                           139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940, 139.940]
        in_drone_lon[2] = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950,
                           139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950]


        in_target_lat[0] = [37.525, 37.515, 37.520, 37.520, 37.525, 37.515, 37.520, 37.520, 37.520, 37.520,
                            37.520, 37.520, 37.520, 37.524, 37.524, 37.524, 37.518, 37.518, 37.518, 37.520, 37.520, 37.520,
                            37.520, 37.520]
        in_target_lat[1] = [37.535, 37.525, 37.530, 37.530, 37.535, 37.525, 37.530, 37.530, 37.530, 37.530,
                            37.531, 37.531, 37.531, 37.532, 37.532, 37.532, 37.529, 37.529, 37.529, 37.528, 37.528, 37.528,
                            37.530, 37.530]
        in_target_lat[2] = [37.545, 37.535, 37.540, 37.540, 37.545, 37.535, 37.540, 37.540, 37.540, 37.540,
                            37.542, 37.542, 37.542, 37.540, 37.540, 37.540, 37.540, 37.540, 37.540, 37.536, 37.536, 37.536,
                            37.540, 37.540]
        in_target_lon[0] = [139.930, 139.930, 139.935, 139.925, 139.930, 139.930, 139.935, 139.925, 139.925, 139.925,
                            139.930, 139.930, 139.930, 139.928, 139.928, 139.928, 139.934, 139.934, 139.934, 139.930, 139.930, 139.930,
                            139.930, 139.930]
        in_target_lon[1] = [139.940, 139.940, 139.945, 139.935, 139.940, 139.940, 139.945, 139.935, 139.935, 139.935,
                            139.938, 139.938, 139.938, 139.939, 139.939, 139.939, 139.942, 139.942, 139.942, 139.941, 139.941, 139.941,
                            139.940, 139.940]
        in_target_lon[2] = [139.950, 139.950, 139.955, 139.945, 139.950, 139.950, 139.955, 139.945, 139.945, 139.945,
                            139.946, 139.946, 139.946, 139.950, 139.950, 139.950, 139.950, 139.950, 139.950, 139.952, 139.952, 139.952,
                            139.950, 139.950]

        in_avg_mag = [315, 315, 315, 315, 135, 135, 135, 135, 
                      315, 135, 315, 315, 315, 315, 315, 315,
                      135, 135, 135, 135, 135, 135, 315, 135]



        in_state[0] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "RTL", "Disarm", "Hovering", "Hovering", "Hovering", "Hovering",  "Disarm", "RTL", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Hovering", "Hovering"]
        in_state[1] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Disarm", "Hovering", "Hovering", "RTL", "Hovering", "Hovering", "Disarm", "Hovering",
                         "Hovering", "RTL", "Hovering", "Hovering"]
        in_state[2] = ["Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Hovering", "Hovering", "Hovering", "Hovering", "RTL", "RTL", "Hovering", "Hovering", "Hovering", "Hovering",
                         "Disarm", "Disarm","Hovering", "Hovering"]


        target_yaw_result[0] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 0, 315], [0, 135],
                                [-1, 296], [ 0, 315], [ 0, 315], [ 1, 333], [ 1, 333], [1, 0],   [-1, 116], [ 0, 135], [ 0, 135],  [1, 153], 
                                [ 1, 153], [ 1, 180], [ 0, 315], [ 0, 135]]
        target_yaw_result[1] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 0, 315], [0, 135],
                                [-1, 296], [-1, 296], [ 0, 315], [ 1, 333], [ 1, 333], [0, 315],   [-1, 116], [-1, 116], [0, 135],  [1, 153], 
                                [ 1, 153], [ 0, 135], [ 0, 315], [ 0, 135]]
        target_yaw_result[2] = [[ 1, 0],   [-1, 180], [ 1, 90],  [-1, 270], [-1, 0],   [1, 180], [-1, 90],  [ 1, 270], [ 0, 315], [0, 135],
                                [-1, 296], [-1, 296], [-1, 270], [ 1, 333], [ 0, 315], [0, 315],   [-1, 116], [-1, 116], [-1, 90],  [1, 153], 
                                [ 0, 135], [ 0, 135], [ 0, 315], [ 0, 135]]

        test_case = 0
        while test_case < len(target_yaw_result[0]):
            xx._turn_dir.return_value = in_turn_dir[test_case]
            self.m._mode = in_mode[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._leader_pos.latitude = in_leader_lat[test_case]
            self.m._leader_pos.longitude = in_leader_lon[test_case]
            self.m._leader_target.latitude = in_leader_target_lat[test_case]
            self.m._leader_target.longitude = in_leader_target_lon[test_case]
            self.m._right_pivot = in_right_pivot[test_case]
            self.m._left_pivot = in_left_pivot[test_case]
            self.m._turn_end = in_turn_end[test_case]
            self.m._avg_mag = in_avg_mag[test_case]


            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_DroneGPSIn[drone_num].latitude = in_drone_lat[drone_num][test_case]
                self.m._d_DroneGPSIn[drone_num].longitude = in_drone_lon[drone_num][test_case]
                self.m._d_TargetGPSOut[drone_num].latitude = in_target_lat[drone_num][test_case]
                self.m._d_TargetGPSOut[drone_num].longitude = in_target_lon[drone_num][test_case]
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]

                self.m._d_outTargetYaw[drone_num].data = init_target_yaw[drone_num][test_case]
                drone_num += 1

            self.m._set_target_yaw()

            drone_num = 0
            while drone_num < DRONE_NUM:
                if self.m._d_outTargetYaw[drone_num].data[1] == 360:
                    self.m._d_outTargetYaw[drone_num].data[1] = 0
                self.assertEqual(self.m._d_outTargetYaw[drone_num].data, target_yaw_result[drone_num][test_case])
                drone_num += 1

            test_case += 1


    def test_is_landing(self):

        in_state = [0] * INPORT_SET_NUM

        in_state[0] = ["RTL", "RTL", "RTL", "RTL", "Hovering", "Hovering", "Disarm", "Armd"]
        in_state[1] = ["RTL", "RTL", "Hovering", "Hovering", "RTL", "RTL", "Disarm", "Armd"]
        in_state[2] = ["RTL", "Hovering", "RTL", "Hovering", "RTL", "Disarm", "RTL", "Disarm"]


        landing_result = [True, True, True, True, True, True, True, False]


        test_case = 0
        while test_case < len(landing_result):

            drone_num = 0
            while drone_num < DRONE_NUM:
                self.m._d_inState[drone_num].data = in_state[drone_num][test_case]
                drone_num += 1

            

            self.assertEqual(self.m._is_landing(), landing_result[test_case])

            test_case += 1

    def test_takeoff_range_check(self):
        in_takeoff_alt = [0] * INPORT_SET_NUM


        in_takeoff_alt[0] = [ 1.0,  1.0,  1.0,  2.0,  3.0, 14.0, 15.0, 16.0, 16.0, 16.0,
                             29.0, 29.0, 29.0, 30.0, 31.0, 49.0, 50.0, 51.0, 51.0, 51.0]
        in_takeoff_alt[1] = [ 1.0,  1.0,  2.0,  2.0,  3.0, 14.0, 15.0, 15.0, 16.0, 16.0,
                             29.0, 29.0, 30.0, 30.0, 31.0, 49.0, 50.0, 50.0, 51.0, 51.0]
        in_takeoff_alt[2] = [ 1.0,  2.0,  2.0,  2.0,  3.0, 14.0, 15.0, 15.0, 15.0, 16.0,
                             29.0, 30.0, 30.0, 30.0, 31.0, 49.0, 50.0, 50.0, 50.0, 51.0]

        in_min_alt = [ 2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,  2.0,
                      30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0, 30.0]
        in_max_alt = [15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0, 15.0,
                      50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0, 50.0]



        result = [False, False, False, True, True, True, True, False, False, False,
                  False, False, False, True, True, True, True, False, False, False]


        test_case = 0
        while test_case < len(result):
            self.m._MinAlt[0] = in_min_alt[test_case]
            self.m._MaxAlt[0] = in_max_alt[test_case]

            drone_num = 0
            while drone_num < DRONE_NUM:
                #
                #self.m._takeoff_alt[drone_num] = in_takeoff_alt[drone_num][test_case]
                self.m._Drone1TakeoffAlt[0] = in_takeoff_alt[0][test_case]
                self.m._Drone2TakeoffAlt[0] = in_takeoff_alt[1][test_case]
                self.m._Drone3TakeoffAlt[0] = in_takeoff_alt[2][test_case]
                #
                drone_num += 1

            ret = self.m._takeoff_range_check()

            self.assertEqual(ret, result[test_case])

            test_case += 1


if __name__ == '__main__':
    unittest.main()
