#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import target_gps

import sys
import math
from collections import namedtuple

from mock import Mock
from mock import patch, MagicMock


class TestTargetGPS(unittest.TestCase):
    def setUp(self):
        self.m = target_gps.TargetGPS()
        
        xx = Mock()
        target_gps.logger.warning = xx.warning


    def test_move_turn_right_lat(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        right_lat_result = [37.517, 37.546, 37.528, 37.533, 37.545, 37.510]
       

        test_case = 0
        while test_case < len(right_lat_result):
            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lat = round(self.m._move_turn_right_lat(p, d, in_mag[test_case]), 3)

            self.assertEqual(lat, right_lat_result[test_case])

            test_case += 1

    def test_move_turn_right_lon(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        right_lon_result = [139.932, 139.965, 139.940, 139.959, 139.978, 139.925]
       

        test_case = 0
        while test_case < len(right_lon_result):
            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lon = round(self.m._move_turn_right_lon(p, d, in_mag[test_case]), 3)

            self.assertEqual(lon, right_lon_result[test_case])

            test_case += 1



    def test_move_turn_left_lat(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        left_lat_result = [37.524, 37.533, 37.532, 37.527, 37.535, 37.531]
       

        test_case = 0
        while test_case < len(left_lat_result):
            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lat = round(self.m._move_turn_left_lat(p, d, in_mag[test_case]), 3)

            self.assertEqual(lat, left_lat_result[test_case])

            test_case += 1

    def test_move_turn_left_lon(self):

        pivot = namedtuple("pivot", ["latitude", "longitude"])
        drone = namedtuple("drone", ["latitude", "longitude"])

        in_pivot_lat = [37.530, 37.530, 37.530, 37.530, 37.530, 37.530]
        in_pivot_lon = [139.950, 139.950, 139.950, 139.950, 139.950, 139.950]
        in_drone_lat = [37.520, 37.520, 37.530, 37.530, 37.540, 37.540]
        in_drone_lon = [139.930, 139.930, 139.940, 139.940, 139.980,139.980]
        in_mag = [350, 160, 350, 160, 350, 160]

        left_lon_result = [139.929, 139.972, 139.940, 139.959, 139.981, 139.918]
       

        test_case = 0
        while test_case < len(left_lon_result):
            p = pivot(in_pivot_lat[test_case], in_pivot_lon[test_case])
            d = drone(in_drone_lat[test_case], in_drone_lon[test_case])
            lon = round(self.m._move_turn_left_lon(p, d, in_mag[test_case]), 3)

            self.assertEqual(lon, left_lon_result[test_case])

            test_case += 1
if __name__ == '__main__':
    unittest.main()
